#!/bin/python3

import json
import requests as r
import re
from urllib import parse
import time
try:
    from ping3 import ping
    def test_connect(url, timeout):
        return ping(url, timeout=timeout)
except:
    from requests import head
    def test_connect(url, timeout):
        _s = time.time()
        try:
            res = head(url, **q)
            return time.time() - _s
        except:
            return None

LOGIN_LOCATION_RE = re.compile("location.href='(?P<login_location>.*)'")

ZWU_AUTH_HOST = 'http://sam.zwu.edu.cn'
SERVICE_INTERNET = 'default'
SERVICE_UNINTERNET = '%E6%A0%A1%E5%9B%AD%E7%BD%91'

from zwu_login import ZwuLogin
from zwu_online_user_info import ZwuOnlineUserInfo



class ZWUAuthAnti:

    user_info: dict = {}


    def __init__(self, login_location=None):

        self.session = r.session()
        self.login_location = login_location


    def _post(self, url, **p):

        res = self.session.post(url, **p)

        res.encoding = 'utf-8'

        return res

    def _get(self, url, **p):

        res = self.session.get(url, **p)

        res.encoding = 'utf-8'

        return res

    def get_login_location(self, force: bool = False):

        if self.login_location and not force:
            return self.login_location


        res = self._get(ZWU_AUTH_HOST)

        login_location_re_res = LOGIN_LOCATION_RE.search(res.text)

        self.login_location = login_location_re_res.group('login_location') if login_location_re_res else None


        return self.login_location

    def query_args(self, key: str):
        if self.login_location == None:
            return None

        _query = parse.parse_qs(parse.urlsplit(self.login_location).query)

        _query_res = _query.get(key)


        return _query_res[0] if _query_res else None

    def get_error_msg(self):
        uri = '/eportal/userV2.do?method=getErrorMsg'

        form_data = {
                'userIndex': self.user_info.get('userIndex'),
                }

        return self._post(ZWU_AUTH_HOST + uri, data=form_data)

    def login(self, user: str, password: str, access_internet: bool = False, valid_code: str = ''):
        uri = '/eportal/InterFace.do?method=login'

        self.get_login_location()
        
        form_data = {
                'userId': user,
                'password': password, # unimplmented
                'queryString': str(parse.urlsplit(self.login_location).query),
                'service': SERVICE_INTERNET if access_internet else SERVICE_UNINTERNET,
                'validcode': valid_code,
                'passwordEncrypt': 'true',
                }


        res = self._post(ZWU_AUTH_HOST + uri, data=form_data)

        self.user_info.clear()

        return ZwuLogin(res.json())

    def fresh_online_user_info(self):
        uri = '/eportal/InterFace.do?method=freshOnlineUserInfo'

        form_data = {
                'userIndex': self.user_info.get('userIndex'),
                }


        return self._post(ZWU_AUTH_HOST + uri, data=form_data)

    def get_online_user_info(self):
        uri = '/eportal/InterFace.do?method=getOnlineUserInfo'

        if self.user_info.get('userIndex') == None:
            self.user_info.update(self._post(ZWU_AUTH_HOST + uri, data={}).json())

        form_data = {
                'userIndex': self.user_info.get('userIndex'),
                }


        self.user_info.clear()
        self.user_info.update(self._post(ZWU_AUTH_HOST + uri, data=form_data).json())

        return ZwuOnlineUserInfo(self.user_info)




    def switch_service(self, service: str):
        uri = '/eportal/InterFace.do?method=switchService'

        form_data = {
                'userIndex': self.user_info.get('userIndex'),
                'serviceName': service,
                }


        return self._post(ZWU_AUTH_HOST + uri, data=form_data)

    def logout(self):
        uri = '/eportal/InterFace.do?method=logout'

        form_data = {
                'userIndex': self.user_info.get('userIndex'),
                }

        return self._post(ZWU_AUTH_HOST + uri, data=form_data)




def main():
    antier = ZWUAuthAnti()

    try:
        while True:
            try:
                res_1 = antier.login('', '', True)
                print(f'bootstrap_url: {antier.login_location}')
                print(res_1)
                time.sleep(0.5)
                if res_1.message == '不在认证时段内!':
                    now = time.localtime()
                    left = 60.0 * 30
                    while left > 0:
                        time.sleep(0.5)
                        now = time.localtime()
                        print(f'\x1b[K{now.tm_hour:02}:{now.tm_min:02}:{now.tm_sec:02} {left:.02f}s', end='\r', flush=True)
                        left -= 0.5
                    time.sleep(0.5)
                    continue
                elif res_1.message == '验证码不正确!':
                    antier = ZWUAuthAnti()
                    left = 60.0
                    while left > 0:
                        time.sleep(0.5)
                        now = time.localtime()
                        print(f'\x1b[K{now.tm_hour:02}:{now.tm_min:02}:{now.tm_sec:02} {left:.02f}s', end='\r', flush=True)
                        left -= 0.5
                    time.sleep(0.5)
                    continue
                res_2 = antier.get_online_user_info()
                print(res_2)
                res_3 = antier.get_error_msg().text
                if res_2.result != 'success':
                    time.sleep(0.5)
                    continue
                #input('!!!')
                #time.sleep(2)
                '''
                now = time.localtime()
                while not [now.tm_hour, now.tm_min, now.tm_sec] == [7, 0, 0]:
                    time.sleep(0.5)
                    now = time.localtime()
                    print(f'\x1b[K{now.tm_hour:02}:{now.tm_min:02}:{now.tm_sec:02}', end='\r', flush=True)
                '''
                while True:
                    now = time.localtime()
                    timer_zwu = test_connect('zwu.edu.cn', timeout=2)
                    timer_bing = test_connect('www.bing.com', timeout=2)
                    if timer_zwu is None:
                        break
                    if timer_bing is None:
                        res = antier.switch_service(SERVICE_INTERNET)
                        print(res.text)
                        continue
                    timer_zwu = int(timer_zwu * 1000)
                    timer_bing = int(timer_bing * 1000)
                    print(f'\x1b[K{now.tm_hour:02}:{now.tm_min:02}:{now.tm_sec:02} [{timer_zwu}|{timer_bing}]ms', end='\r', flush=True)
                    time.sleep(0.5)
                antier.logout()
            except KeyboardInterrupt:
                break
            except Exception as e:
                raise e
    except KeyboardInterrupt:
        pass
    finally:
        antier.logout()


if __name__ == '__main__':

    main()
